package com.tocea.easycoverage.framework.junit;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import junit.framework.TestCase;
import junit.framework.TestResult;

import com.tocea.easycoverage.framework.api.IEasycoverageChecker;

public class EasycoverageTestCase extends TestCase {

	private final IEasycoverageChecker	checker;
	private final ExecutorService		executorService;
	private final int					timeout;

	public EasycoverageTestCase(final IEasycoverageChecker checker,
			final ExecutorService executorService, final int timeout) {
		this.checker = checker;
		this.executorService = executorService;
		this.timeout = timeout;
		setName(checker.toString());
	}

	@Override
	public void run(final TestResult result) {
		result.startTest(this);

		final Future<?> future = executorService.submit(new TestTask());
		try {
			if (timeout > 0)
				future.get(timeout, TimeUnit.MILLISECONDS);
			else
				future.get();
		} catch (final InterruptedException e) {
			// oops
		} catch (final ExecutionException e) {
			result.addError(this, e.getCause());
		} catch (final TimeoutException e) {
			if (future != null && !future.isDone() && !future.isCancelled())
				future.cancel(true);
			result.addError(this, new Exception(getName()
					+ " has timed out after " + timeout + "ms.", e));
		} finally {
			result.endTest(this);
		}
	}

	private class TestTask implements Runnable {

		public void run() {
			checker.check();
		}

	}

}
